if (!sape) var sape={};
if (!sape.toolbar) sape.toolbar={};

Components.utils['import']("resource://gre/modules/Services.jsm");

if (!sape.toolbar.cache) sape.toolbar.cache =
{
    lifetime: 24,
    historytime: 3,
    enabled: true
};

sape.toolbar.cache.init = function()
{
    var me = sape.toolbar.cache;

    if (!me.hasOwnProperty('prefs'))
    {
        me.prefs = Services.prefs.getBranch("extensions.wmsape.");
    }
    me.enabled      = me.prefs.getBoolPref("cache");
    me.historytime  = me.prefs.getIntPref("history.age");
    me.lifetime     = me.prefs.getIntPref("cache.lifetime");

    me.prefs.addObserver("", sape.toolbar.cache.observer, false);
    window.addEventListener("unload", sape.toolbar.cache.done, false);

    me.db.init();
};

sape.toolbar.cache.observer =
{
    observe: function(aSubject, aTopic, aData)
    {
        if (sape && aTopic == "nsPref:changed")
        {
            var me = sape.toolbar.cache;

            if (aData.indexOf("cache") > -1)            me.enabled      = me.prefs.getBoolPref("cache");
            if (aData.indexOf("history.age") > -1)      me.historytime  = me.prefs.getIntPref("history.age");
            if (aData.indexOf("cache.lifetime") > -1)   me.lifetime     = me.prefs.getIntPref("cache.lifetime");
        }
    }
};

sape.toolbar.cache.done = function()
{
    var me = sape.toolbar.cache;

    if (me.db.dbc) me.db.cleanup();
    window.removeEventListener("unload", sape.toolbar.cache.done, false);
};

sape.toolbar.cache.push =  function(url, type, value)
{
    var me = sape.toolbar.cache;
    if (!me.db.dbc) me.db.init();

    var callback = (arguments.length > 3)? arguments[3] : null;
    if (me.db.dbc)
    {
        me.db.push(url, type, value, callback);
    }
    else
    {
        if (callback) callback();
    }
};

sape.toolbar.cache.pop = function(url, type)
{
    var me = sape.toolbar.cache;
    if (!me.db.dbc) me.db.init();

    var callback = (arguments.length > 2)? arguments[2] : null;
    if (me.db.dbc)
    {
        if (callback)
        {
            me.db.popAsync(url, type, callback);
        }
        else
        {
            return me.db.pop(url, type);
        }
    }
    return null;
};

sape.toolbar.cache.del = function(url, type)
{
    var me = sape.toolbar.cache;
    if (!me.db.dbc) me.db.init();

    if (me.db.dbc) me.db.del(url, type);
};

sape.toolbar.cache.type = function(type, callback)
{
    var me = sape.toolbar.cache;
    if (!me.db.dbc) me.db.init();

    if (me.db.dbc)
    {
        me.db.get_by_type(type, callback);
    }
    else
    {
        if (callback) callback([]);
    }
};

sape.toolbar.cache.deltype = function(type, callback)
{
    var me = sape.toolbar.cache;
    if (!me.db.dbc) me.db.init();

    if (me.db.dbc)
    {
        me.db.del_by_type(type, callback);
    }
    else
    {
        if (callback) callback();
    }
};

sape.toolbar.cache.age = function(url, type, callback)
{
    var me = sape.toolbar.cache;
    if (!me.db.dbc) me.db.init();

    if (me.db.dbc)
    {
        me.db.age(url, type, callback);
    }
    else
    {
        if (callback) callback(0);
    }
};

//------------------------------------------------------------------------------
// Кэширование на основе базы данных
//------------------------------------------------------------------------------

if (!sape.toolbar.cache.db) sape.toolbar.cache.db =
{
    dbc: null
};

sape.toolbar.cache.db.init = function()
{
    var cache = sape.toolbar.cache, me = cache.db, e;
	if (me.dbc) return;

    try
    {
        var dbfile = Components.classes["@mozilla.org/file/directory_service;1"].getService(Components.interfaces.nsIProperties).get("ProfD", Components.interfaces.nsIFile);
    	    dbfile.append("wmsape.sqlite");

    	var storageService = Components.classes["@mozilla.org/storage/service;1"].getService(Components.interfaces.mozIStorageService);
        try
        {
            me.dbc = storageService.openDatabase(dbfile);
            me.dbc.createStatement("CREATE TABLE IF NOT EXISTS base(type VARCHAR(8), len INTEGER, url LONGVARCHAR, value LONGVARCHAR, up DATETIME)").executeAsync(
            {
                handleCompletion: function(aReason)
                {
                    me.dbc.createStatement("CREATE INDEX IF NOT EXISTS cmp_idx ON base(type, len)").executeAsync();
                    me.dbc.createStatement("CREATE INDEX IF NOT EXISTS up_idx ON base(up)").executeAsync();
                    me.dbc.createStatement("PRAGMA journal_mode = WAL").executeAsync();
                }
            });
        }
        catch (e if e.result == Components.results.NS_ERROR_FILE_CORRUPTED)
        {
            if (dbfile) dbfile.remove(false);
            throw e;
        }
    }
    catch(e){}
};

sape.toolbar.cache.db.cleanup = function()
{
    var cache = sape.toolbar.cache, me = cache.db;
	if (me.dbc)
	{
        var lifetime = cache.lifetime + 24;
        var historytime  = cache.historytime * 24 + 240;
        try
        {
            var statement1 = me.dbc.createStatement('DELETE FROM base WHERE datetime(up) < datetime("now", "localtime", ?1) AND `type` <> "history"');
                statement1.bindUTF8StringParameter(0, "-" + lifetime.toString() + " hour");
                statement1.executeAsync();

            var statement2 = me.dbc.createStatement('DELETE FROM base WHERE datetime(up) < datetime("now", "localtime", ?1) AND `type` = "history"');
                statement2.bindUTF8StringParameter(0, "-" + historytime.toString() + " hour");
                statement2.executeAsync();

        	me.dbc.asyncClose();
        }
        catch(e) {}
	}
};

sape.toolbar.cache.db.drop = function()
{
    var cache = sape.toolbar.cache, me = cache.db;
	if (!me.dbc) me.init();

	if (me.dbc)
	{
	    me.dbc.createStatement('DELETE FROM base WHERE `type` <> "history"').executeAsync();
        alert('Cache cleanup completed');
	}
};

sape.toolbar.cache.db.hdrop = function()
{
    var cache = sape.toolbar.cache, me = cache.db;
	if (!me.dbc) me.init();

	if (me.dbc)
	{
	    me.dbc.createStatement('DELETE FROM base WHERE `type` = "history"').executeAsync();
        alert('History cleanup completed');
	}
};

sape.toolbar.cache.db.push = function(url, type, value, callback)
{
    var me = sape.toolbar.cache.db;
	if (me.dbc)
	{
        try
        {
           var statement1 = me.dbc.createStatement("DELETE FROM base WHERE type=?1 AND len=?2 AND url=?3");
               statement1.bindInt32Parameter(1, url.length);
               statement1.bindUTF8StringParameter(0, type);
               statement1.bindUTF8StringParameter(2, url);

           var statement2 = me.dbc.createStatement("INSERT INTO base(type, len, url, value, up) VALUES (?1, ?2, ?3, ?4, datetime('now', 'localtime'))");
               statement2.bindInt32Parameter(1, url.length);
               statement2.bindUTF8StringParameter(0, type);
               statement2.bindUTF8StringParameter(2, url);
               statement2.bindUTF8StringParameter(3, value);

           var s2handle = {
               handleCompletion: function(aReason)
               {
                    if (callback) callback();
               }
           };

           var s1handle = {
               handleCompletion: function(aReason)
               {
                    statement2.executeAsync(s2handle);
               }
           };

           statement1.executeAsync(s1handle);
        }
        catch(e) {}
    }
    else
    {
        if (callback) callback();
    }
};

sape.toolbar.cache.db.del = function(url, type)
{
    var me = sape.toolbar.cache.db;
	if (me.dbc)
	{
        try
        {
            var statement = me.dbc.createStatement("DELETE FROM base WHERE type=?1 AND len=?2 AND url=?3");
                statement.bindInt32Parameter(1, url.length);
                statement.bindUTF8StringParameter(0, type);
                statement.bindUTF8StringParameter(2, url);
                statement.executeAsync();
        }
        catch(e) {}
    }
};

sape.toolbar.cache.db.pop = function(url, type)
{
    var me       = sape.toolbar.cache.db;
    var lifetime = sape.toolbar.cache.lifetime;
	if (me.dbc)
	{
        try
        {
			var statement = me.dbc.createStatement("SELECT value, up FROM base WHERE type=?1 AND len=?2 AND url=?3");
                statement.bindUTF8StringParameter(0, type);
                statement.bindUTF8StringParameter(2, url);
                statement.bindInt32Parameter(1, url.length);

			while (statement.step())
			{
				var v = statement.getString(0);
                var t = statement.getString(1).match(/([0-9]+)\-([0-9]+)\-([0-9]+)\s([0-9]+)\:([0-9]+)\:([0-9]+)/);

                var cacheDate = new Date(t[1],t[2]-1,t[3],t[4],t[5],t[6],0);
                var lastDate =  new Date();

                lastDate.setHours(lastDate.getHours() - lifetime);
                if (cacheDate > lastDate)
                {
                    statement.reset();
                    return isFinite(v)? +v : v;
                }
            }
        }
        catch(e) {}
    }
    return null;
};


sape.toolbar.cache.db.popAsync = function(url, type, callback)
{
    var cache = sape.toolbar.cache, me = cache.db, result = null, lifetime = cache.lifetime;
	if (me.dbc)
	{
	    var statement = me.dbc.createStatement("SELECT value, up FROM base WHERE type=?1 AND len=?2 AND url=?3");
            statement.bindUTF8StringParameter(0, type);
            statement.bindUTF8StringParameter(2, url);
            statement.bindInt32Parameter(1, url.length);

        statement.executeAsync(
        {
            handleResult: function(aResultSet)
            {
                var row = aResultSet.getNextRow();
                if (row)
                {
                    var v = row.getResultByName('value');
                    var t = row.getResultByName('up');

                    var tm = t.match(/([0-9]+)\-([0-9]+)\-([0-9]+)\s([0-9]+)\:([0-9]+)\:([0-9]+)/);
                    if (tm)
                    {
                        var cacheDate = new Date(tm[1], tm[2]-1, tm[3], tm[4], tm[5], tm[6], 0);
                        var lastDate =  new Date();

                        lastDate.setHours(lastDate.getHours() - lifetime);
                        if (cacheDate > lastDate)
                        {
                            result = isFinite(v)? +v : v;
                        }
                    }
                }
            },
            handleCompletion: function(aReason)
            {
                if (callback) callback(result);
            }
        });
    }
    else
    {
        callback(null);
    }
};

sape.toolbar.cache.db.get_by_type = function(type, callback)
{
    var me = sape.toolbar.cache.db, result = [];
	if (me.dbc)
	{
	    var statement = me.dbc.createStatement("SELECT url,value,up FROM base WHERE type LIKE ?1 ORDER BY up DESC");
            statement.bindUTF8StringParameter(0, type);

        statement.executeAsync(
        {
            handleResult: function(aResultSet)
            {
                for (var row = aResultSet.getNextRow(); row; row = aResultSet.getNextRow())
                {
                    result[result.length] =
                    {
                        key:   row.getResultByName('url'),
                        value: row.getResultByName('value'),
                        time:  row.getResultByName('up')
                    };
                }
            },
            handleCompletion: function(aReason)
            {
                if (callback) callback(result);
            }
        });
    }
    else
    {
        if (callback) callback(result);
    }
};

sape.toolbar.cache.db.del_by_type = function(type, callback)
{
    var me = sape.toolbar.cache.db;
	if (me.dbc)
	{
	    var statement = me.dbc.createStatement("DELETE FROM base WHERE type LIKE ?1");
            statement.bindUTF8StringParameter(0, type);
            statement.executeAsync(
            {
                handleCompletion: function(aReason)
                {
                    if (callback) callback();
                }
            });
    }
    else
    {
        if (callback) callback();
    }
};

// Возвращает в callback возраст записи в секундах либо null если записи нет
sape.toolbar.cache.db.age = function(url, type, callback)
{
    var me = sape.toolbar.cache.db, result = 0;
	if (me.dbc)
	{
    	var statement = me.dbc.createStatement("SELECT up FROM base WHERE type=?1 AND len=?2 AND url=?3");
            statement.bindUTF8StringParameter(0, type);
            statement.bindUTF8StringParameter(2, url);
            statement.bindInt32Parameter(1, url.length);

        statement.executeAsync(
        {
            handleResult: function(aResultSet)
            {
                var row = aResultSet.getNextRow();
                var up  = row.getResultByName('up');
                var tm  = up.match(/([0-9]+)\-([0-9]+)\-([0-9]+)\s([0-9]+)\:([0-9]+)\:([0-9]+)/);
                if (tm)
                {
                    var cacheDate = new Date(tm[1], tm[2]-1, tm[3], tm[4], tm[5], tm[6], 0);
                    var nowDate   = new Date();

                    result = (nowDate - cacheDate) / 1000;
                }
            },
            handleCompletion: function(aReason)
            {
                if (callback) callback(result);
            }
        });
    }
    else
    {
        if (callback) callback(result);
    }
};
