var sape = sape || {};

if (!sape.history) sape.history = {};

sape.history.parsers =
{
    common: function(datarow)
    {
        var result = [], parts = datarow.key.split('++');

        result[result.length] = parts[0];
        result[result.length] = datarow.value;

        return result;
    },
    ymt: function(datarow)
    {
        var result = [], parts = datarow.key.split('++');

        var exp = /(\d{4})(\d{2})(\d{2})T(\d{2})(\d{2})(\d{2})/, S = datarow.value;
        if ((exp = exp.exec(S)))
        {
            S = exp[1] + '-' + exp[2] + '-' + exp[3] + ' ' + exp[4] + ':' + exp[5] + ':' + exp[6];
        }
        result[result.length] = parts[0];
        result[result.length] = S;

        return result;
    },
    linktext: function(datarow)
    {
        var result = [], parts = datarow.key.split('++');

        result[result.length] = parts[0];
        result[result.length] = (parts.length > 1)? parts[1] : "";
        result[result.length] = datarow.value;

        return result;
    },
    pagesize: function(datarow)
    {
        var result = [], parts = datarow.key.split('++'), jdata = JSON.parse(datarow.value);

        result[result.length] = parts[0];
        result[result.length] = jdata.ps || '-';
        result[result.length] = jdata.ts || '-';

        return result;
    },
    pagelinks: function(datarow)
    {
        var result = [], parts = datarow.key.split('++'), jdata = JSON.parse(datarow.value);

        result[result.length] = parts[0];
        result[result.length] = jdata.allext || '0';
        result[result.length] = jdata.allnext || '0';

        return result;
    },
    tic: function(datarow)
    {
        var result = [], parts = datarow.key.split('++'), jdata = JSON.parse(datarow.value);

        result[result.length] = parts[0];
        result[result.length] = jdata.domain || '-';
        result[result.length] = jdata.tic || '0';
        result[result.length] = (isFinite(jdata.rang) && isFinite(jdata.tic) && !jdata.tic && (+jdata.rang > +jdata.tic))? '1' : '0';

        return result;
    },
    moz: function(datarow)
    {
        var result = [], parts = datarow.key.split('++'), jdata = JSON.parse(datarow.value);

        result[result.length] = parts[0];
        result[result.length] = parseFloat(jdata.umrp).toFixed(2);
        result[result.length] = parseFloat(jdata.upa).toFixed(2);
        result[result.length] = parseFloat(jdata.fmrp).toFixed(2);
        result[result.length] = parseFloat(jdata.pda).toFixed(2);

        return result;
    },
    flow: function(datarow)
    {
        var result = [], parts = datarow.key.split('++'), jdata = JSON.parse(datarow.value);

        result[result.length] = parts[0];
        result[result.length] = jdata.CitationFlow || '0';
        result[result.length] = jdata.TrustFlow || '0';
        result[result.length] = jdata.RefDomains || '0';
        result[result.length] = jdata.ExtBackLinks || '0';

        return result;
    },
    checktrust: function(datarow)
    {
        var result = [], parts = datarow.key.split('++'), jdata = JSON.parse(datarow.value);

        result[result.length] = parts[0];
        result[result.length] = jdata.trust || 'n/a';
        result[result.length] = jdata.spam || 'n/a';

        return result;
    },
    xtool: function(datarow)
    {
        var result = [], parts = datarow.key.split('++'), jdata = JSON.parse(datarow.value), st = sape.toolbar;

        result[result.length] = parts[0];
        result[result.length] = isFinite(jdata.xtd)? jdata.xtd : 'n/a';
        result[result.length] = isFinite(jdata.xt)? jdata.xt : 'n/a';
        result[result.length] = isFinite(jdata.change)? jdata.change : '';
        result[result.length] = isFinite(jdata.spam)? jdata.spam : 'n/a';
        result[result.length] = (isFinite(jdata.sanctions) && (jdata.sanctions > 0) && (jdata.sanctions < 5))? st.str('xtool.sanctions.' + jdata.sanctions) : '';

        return result;
    },
    lpad: function(datarow)
    {
        var result = [], parts = datarow.key.split('++'), jdata = JSON.parse(datarow.value);

        result[result.length] = parts[0];
        result[result.length] = jdata.pages || '0';
        result[result.length] = jdata.hin || '0';
        result[result.length] = jdata.hout || '0';
        result[result.length] = jdata.din || '0';
        result[result.length] = jdata.dout || '0';

        return result;
    },
    social: function(datarow)
    {
        var result = [], parts = datarow.key.split('++'), jdata = JSON.parse(datarow.value);

        result[result.length] = parts[0];
        result[result.length] = isFinite(jdata.vkontakte)? jdata.vkontakte : '-';
        result[result.length] = isFinite(jdata.facebook)? jdata.facebook : '-';
        result[result.length] = isFinite(jdata.odnoklassniki)? jdata.odnoklassniki : '-';
        result[result.length] = isFinite(jdata.moimir)? jdata.moimir : '-';
        result[result.length] = isFinite(jdata.gplus)? jdata.gplus : '-';
        result[result.length] = isFinite(jdata.linkedin)? jdata.linkedin : '-';
        result[result.length] = isFinite(jdata.pinterest)? jdata.pinterest : '-';
        result[result.length] = isFinite(jdata.stumbleupon)? jdata.stumbleupon : '-';

        return result;
    },
    ahrefs: function(datarow)
    {
        var result = [], parts = datarow.key.split('++'), jdata = JSON.parse(datarow.value);

        result[result.length] = parts[0];
        result[result.length] = isFinite(jdata.AR)? jdata.AR : '-';
        result[result.length] = isFinite(jdata.UR)? jdata.UR : '-';
        result[result.length] = isFinite(jdata.BL)? jdata.BL : '-';
        result[result.length] = isFinite(jdata.RD)? jdata.RD : '-';
        result[result.length] = isFinite(jdata.DDR)? jdata.DDR : '-';
        result[result.length] = isFinite(jdata.DBL)? jdata.DBL : '-';
        result[result.length] = isFinite(jdata.DRD)? jdata.DRD : '-';

        return result;
    }
};

sape.history.farm =
{
    host:
    {
        name: "yandex-sites", filter: "host.%", view: 'common', header: [['%3%', '75%'], ['%4%', '15%'], ['%1%', '10%']]
    },
    yap:
    {
        name: "yandex-pages", filter: "yap.%", view: 'common', header: [['%2%', '75%'], ['%6%', '15%'], ['%1%', '10%']]
    },
    yal:
    {
        name: "yandex-links", filter: "yal.%", view: 'linktext', header: [['%2%', '50%'], ['%5%', '30%'], ['%6%', '10%'], ['%1%', '10%']]
    },
    yacl:
    {
        name: "yandex-links-cache", filter: "yacl", view: 'linktext', header: [['%2%', '50%'], ['%5%', '30%'], ['%6%', '10%'], ['%1%', '10%']]
    },
    his:
    {
        name: "yandex-noindex", filter: "history", view: 'linktext', header: [['%2%', '50%'], ['%5%', '30%'], ['%6%', '10%'], ['%1%', '10%']]
    },
    yac:
    {
        name: "yandex-cache", filter: "yac", view: 'common', header: [['%2%', '50%'], ['%7%', '40%'], ['%1%', '10%']]
    },
    yacd:
    {
        name: "yandex-cache-date", filter: "yacd", view: 'common', header: [['%2%', '75%'], ['%8%', '15%'], ['%1%', '10%']]
    },
    ymt:
    {
        name: "yandex-modify-date", filter: "ymt", view: 'ymt', header: [['%2%', '75%'], ['%38%', '15%'], ['%1%', '10%']]
    },
    gs:
    {
        name: "google-sites", filter: "gs", view: 'common', header: [['%3%', '75%'], ['%4%', '15%'], ['%1%', '10%']]
    },
    gp:
    {
        name: "google-pages", filter: "gp", view: 'common', header: [['%2%', '75%'], ['%6%', '15%'], ['%1%', '10%']]
    },
    gm:
    {
        name: "google-pages-main", filter: "gm", view: 'common', header: [['%2%', '75%'], ['%6%', '15%'], ['%1%', '10%']]
    },
    gl:
    {
        name: "google-links", filter: "gl", view: 'linktext', header: [['%2%', '50%'], ['%5%', '30%'], ['%6%', '10%'], ['%1%', '10%']]
    },
    gc:
    {
        name: "google-cache", filter: "gc", view: 'common', header: [['%2%', '75%'], ['%7%', '15%'], ['%1%', '10%']]
    },
    gcd:
    {
        name: "google-cache-date", filter: "gcd", view: 'common', header: [['%2%', '75%'], ['%8%', '15%'], ['%1%', '10%']]
    },
    bs:
    {
        name: "bing-sites", filter: "bings", view: 'common', header: [['%3%', '75%'], ['%4%', '15%'], ['%1%', '10%']]
    },
    bp:
    {
        name: "bing-pages", filter: "bingp", view: 'common', header: [['%2%', '75%'], ['%6%', '15%'], ['%1%', '10%']]
    },
    bl:
    {
        name: "bing-links", filter: "bingl", view: 'linktext', header: [['%2%', '50%'], ['%5%', '30%'], ['%6%', '10%'], ['%1%', '10%']]
    },
    ags:
    {
        name: "yaca-ags", filter: "ags", view: 'common', header: [['%3%', '75%'], ['%9%', '15%'], ['%1%', '10%']]
    },
    pr:
    {
        name: "pagerank", filter: "pr", view: 'common', header: [['%2%', '75%'], ['%10%', '15%'], ['%1%', '10%']]
    },
    tic:
    {
        name: "tic", filter: "tic", view: 'tic', header: [['%2%', '60%'], ['%3%', '20%'], ['%11%', '5%'], ['%9%', '5%'], ['%1%', '10%']]
    },
    moz:
    {
        name: "mozrank", filter: "mozrank", view: 'moz', header: [['%2%', '50%'], ['%13%', '10%'], ['%14%', '10%'], ['%15% %13%', '10%'], ['%15% %14%', '10%'], ['%1%', '10%']]
    },
    flow:
    {
        name: "flow", filter: "flow", view: 'flow', header: [['%2%', '50%'], ['%16%', '10%'], ['%17%', '10%'], ['%18%', '10%'], ['%19%', '10%'], ['%1%', '10%']]
    },
    ahrefs:
    {
        name: "ahrefs", filter: "ahrefs", view: 'ahrefs', header: [['%2%', '45%'], ['Ahrefs Rank', '5%'], ['URL Rating', '5%'], ['Backlinks', '5%'], ['Ref. Domains', '5%'], ['Domain Raiting', '5%'], ['Domain Backlinks', '5%'], ['Domain Ref. Domains', '5%'], ['%1%', '10%']]
    },
    alexa:
    {
        name: "alexa", filter: "alexa", view: 'common', header: [['%21%', '75%'], ['%20%', '15%'], ['%1%', '10%']]
    },
    liru:
    {
        name: "liveinternet", filter: "liru", view: 'common', header: [['%21%', '75%'], ['%22%', '15%'], ['%1%', '10%']]
    },
    similarweb:
    {
        name: "similarweb", filter: "similar", view: 'common', header: [['%21%', '75%'], ['%22%', '15%'], ['%1%', '10%']]
    },
    rush:
    {
        name: "semrush", filter: "rush.%", view: 'common', header: [['%21%', '75%'], ['%23%', '15%'], ['%1%', '10%']]
    },
    unique:
    {
        name: "unique", filter: "unique", view: 'common', header: [['%2%', '75%'], ['%24%', '15%'], ['%1%', '10%']]
    },
    lpad:
    {
        name: "linkpad", filter: "lpad", view: 'lpad', header: [['%21%', '55%'], ['%4%', '10%'], ['%25%', '10%'], ['%26%', '10%'], ['%27%', '10%'], ['%28%', '10%'], ['%1%', '10%']]
    },
    social:
    {
        name: "likes", filter: "like", view: 'social',
        header: [['%2%', '50%'], ['Vk', '5%'], ['Fb', '5%'], ['Ok', '5%'], ['Mir', '5%'], ['GP', '5%'], ['Li', '5%'], ['Pn', '5%'], ['Su', '5%'], ['%1%', '10%']]
    },
    xt:
    {
        name: "xtool", filter: "xt", view: 'xtool', header: [['%3%', '60%'], ['%36%', '5%'], ['%41%', '5%'], ['%40%', '5%'], ['%37%', '5%'], ['%39%', '5%'], ['%1%', '10%']]
    },
    lvl:
    {
        name: "level", filter: "lvl", view: 'common', header: [['%2%', '75%'], ['%29%', '15%'], ['%1%', '10%']]
    },
    lf:
    {
        name: "url-stop-words", filter: "lf", view: 'common', header: [['%2%', '75%'], ['%30%', '15%'], ['%1%', '10%']]
    },
    tf:
    {
        name: "stop-words", filter: "tf", view: 'common', header: [['%2%', '75%'], ['%30%', '15%'], ['%1%', '10%']]
    },
    ps:
    {
        name: "page-size", filter: "ps", view: 'pagesize', header: [['%2%', '70%'], ['%31%', '10%'], ['%32%', '10%'], ['%1%', '10%']]
    },
    ext:
    {
        name: "ext-links", filter: "ext", view: 'common', header: [['%2%', '75%'], ['%33%', '15%'], ['%1%', '10%']]
    },
    allext:
    {
        name: "page-links", filter: "allext", view: 'pagelinks', header: [['%2%', '70%'], ['%34%', '10%'], ['%35%', '10%'], ['%1%', '10%']]
    },
    dnsip:
    {
        name: "ip-", filter: "dnsip", view: 'common', header: [['%3%', '65%'], ['IP', '25%'], ['%1%', '10%']]
    },
    ct:
    {
        name: "checktrust", filter: "ct", view: 'checktrust', header: [['%3%', '70%'], ['%36%', '10%'], ['%37%', '10%'], ['%1%', '10%']]
    }
};

sape.history.getType = function(doc)
{
    var typeSelect = doc.getElementById("typeSelect");
    return (typeSelect)? typeSelect.value : "";
};

sape.history.l11n = function(str)
{
    var r = function(s, i)
    {
        return sape.toolbar.str("history.names." + i);
    };
    return str.replace(/\%(\d+)\%/g, r);
};

sape.history.table =
{
    header: function(data)
    {
        var st = sape.toolbar, me = sape.history, doc = st.getDocument();

        var colgroup = doc.querySelector("#maintable colgroup");
        while (colgroup.firstChild)
        {
            colgroup.removeChild(colgroup.firstChild);
        }
        var head = doc.querySelector("#maintable thead tr");
        while (head.firstChild)
        {
            head.removeChild(head.firstChild);
        }

        for (var i=0; i < data.length; i++)
        {
            var col = doc.createElement("col");
                col.setAttribute("width", data[i][1]);

            var td = doc.createElement("td");
                td.setAttribute("width", data[i][1]);
                td.textContent = me.l11n(data[i][0]);

            colgroup.appendChild(col);
            head.appendChild(td);
        }
    },

    body: function(data, worker)
    {
        var st = sape.toolbar, me = sape.history, doc = st.getDocument();

        var body = doc.getElementById("tablebody");
        while (body.firstChild)
        {
            body.removeChild(body.firstChild);
        }
        var dl = Math.min(data.length, 5000);
        for (var i=0; i < dl; i++)
        {
            var tr = doc.createElement("tr");
            var values = me.parsers[worker.view](data[i]);

            values[values.length] = data[i].time;

            for (var j=0; j< values.length; j++)
            {
                var td = doc.createElement("td");
                    td.textContent = values[j];

                tr.appendChild(td);
            }
            body.appendChild(tr);
        }
    }
};

sape.history.tsv =
{
    header: function(data)
    {
        for (var i=0, parts = []; i < data.length; i++)
        {
            parts[parts.length] = sape.history.l11n(data[i][0]);
        }
        return parts.join("\t");
    },

    body: function(data, worker)
    {
        for (var i=0, lines = []; i < data.length; i++)
        {
            var values = sape.history.parsers[worker.view](data[i]);
                values[values.length] = data[i].time;

            for (var j=0, parts = []; j< values.length; j++)
            {
                parts[parts.length] = values[j];
            }
            lines[lines.length] = parts.join("\t");
        }
        return lines.join("\n");
    }
};

sape.history.save = function()
{
    var st = sape.toolbar, me = sape.history, doc = st.getDocument();

    var type = me.getType(doc);
    if (!type) return;

    var worker = (me.farm.hasOwnProperty(type))? me.farm[type] : null;
    if (!worker) return;

    st.cache.type(worker.filter, function(data)
    {
        var dataString = me.tsv.header(worker.header) + "\n" + me.tsv.body(data, worker);

        var nsIFilePicker = Components.interfaces.nsIFilePicker;

        var fp = Components.classes["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);
            fp.init(window, st.str('fp.saveas'), nsIFilePicker.modeSave);
            fp.defaultString = worker.name + "-export.tsv";
            fp.appendFilter(st.str('fp.tsv'), "*.tsv");
            fp.appendFilter(st.str('fp.excel'), "*.xls");
            fp.appendFilter(st.str('fp.all'), "*.*");

        var res = fp.show();
        if (res == nsIFilePicker.returnOK || res == nsIFilePicker.returnReplace)
        {
           var foStream = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance(Components.interfaces.nsIFileOutputStream);
               foStream.init(fp.file, 0x02 | 0x08 | 0x10, 438, 0);

           var os = Components.classes["@mozilla.org/intl/converter-output-stream;1"].createInstance(Components.interfaces.nsIConverterOutputStream);
               os.init(foStream, "windows-1251", 0, 0);
               os.writeString(dataString);
               os.close();

           foStream.close();
        }
    });
};

sape.history.show = function()
{
    var st = sape.toolbar, me = sape.history, doc = st.getDocument();

    var type = me.getType(doc);
    if (!type) return;

    var worker = (me.farm.hasOwnProperty(type))? me.farm[type] : null;
    if (!worker) return;

    st.cache.type(worker.filter, function(data)
    {
        me.table.header(worker.header);
        me.table.body(data, worker);

        var evt = document.createEvent("HTMLEvents");
            evt.initEvent("change", true, true);

        var maintable = doc.getElementById("maintable");
            maintable.dispatchEvent(evt);

        st.initDOM();
    });
};

sape.history.cleanup = function()
{
    var st = sape.toolbar, me = sape.history, doc = st.getDocument();

    var type = me.getType(doc);
    if (!type) return;

    var worker = (me.farm.hasOwnProperty(type))? me.farm[type] : null;
    if (!worker) return;

    var prompt = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);
    var result = prompt.confirm(null, st.str('history.cleanup.title'), st.str('history.cleanup.message'));

    if (result)
    {
        st.cache.deltype(worker.filter, function()
        {
            sape.history.show();
        });
    }
};

sape.history.change = function(event)
{
    var st = sape.toolbar, typeSelect = event.target;
    if (typeSelect.value)
    {
        st.prefs().setCharPref("exports.last", typeSelect.value);
        st.setTimer(sape.history.show, 100);
    }
};

sape.history.init = function()
{
    var st = sape.toolbar, doc = (arguments.length)? arguments[0] : st.getDocument();

    var showLink = doc.getElementById("showLink");
    if (showLink) showLink.addEventListener("click", sape.history.show, true);

    var exportLink = doc.getElementById("exportLink");
    if (exportLink) exportLink.addEventListener("click", sape.history.save, true);

    var deleteLink = doc.getElementById("deleteLink");
    if (deleteLink) deleteLink.addEventListener("click", sape.history.cleanup, true);

    var typeSelect = doc.getElementById("typeSelect");
    if (typeSelect)
    {
        typeSelect.addEventListener("change", sape.history.change, true);

        var lastValue = st.prefs().getCharPref("exports.last");
        if (lastValue && (lastValue !== typeSelect.value))
        {
            typeSelect.value  = lastValue;
        }
    }
};
