/*
  SortTable
  version 2
  7th April 2007
  Stuart Langridge, http://www.kryogenix.org/code/browser/sorttable/

  version 2.1 (this version)
  25th January 2015
  Wink, reformation code for using in Mozilla addons

  Thanks to many, many people for contributions and suggestions.
  Licenced as X11: http://www.kryogenix.org/code/browser/licence.html
  This basically means: do what you want with it.
*/

var sorttable =
{
    DATE_RE: /^(\d\d?)[\/\.-](\d\d?)[\/\.-]((\d\d)?\d\d)$/,

    init: function()
    {
        // quit if this function has already been called, flag this function so we don't do the same thing twice
        if (arguments.callee.done) return;
        else
            arguments.callee.done = true;

        var tables = document.querySelectorAll('table.sortable');
        for (var i=0; i < tables.length; i++)
        {
            sorttable.makeSortable(tables[i]);
        }
    },

    makeSortable: function(table)
    {
        if (!table.getElementsByTagName('thead').length)
        {
            var the = document.createElement('thead');
                the.appendChild(table.rows[0]);
            table.insertBefore(the, table.firstChild);
        }
        // Safari doesn't support table.tHead, sigh
        if (!table.tHead)
        {
            table.tHead = table.getElementsByTagName('thead')[0];
        }

        if (table.tHead.rows.length != 1) return; // can't cope with two header rows

        // work through each column and calculate its type
        var headrow = table.tHead.rows[0].cells;
        for (var i=0; i < headrow.length; i++)
        {
            // skip this col
            if (headrow[i].className.match(/\bsorttable_nosort\b/)) continue;

            // manually override the type with a sorttable_type attribute
            var mtch = headrow[i].className.match(/\bsorttable_([a-z0-9]+)\b/), override = '';
            if (mtch)
            {
                override = mtch[1];
            }
	        if (mtch && typeof sorttable["sort_" + override] == 'function')
            {
	            headrow[i].sorttable_sortfunction = sorttable["sort_" + override];
	        }
            else
            {
	            headrow[i].sorttable_sortfunction = sorttable.guessType(table,i);
	        }
	        // make it clickable to sort
	        headrow[i].sorttable_columnindex = i;
	        headrow[i].sorttable_tbody = table.tBodies[0];
            headrow[i].addEventListener("click", function(e)
            {
                if (this.className.search(/\bsorttable_sorted\b/) != -1)
                {
                    // if we're already sorted by this column, just reverse the table, which is quicker
                    sorttable.reverse(this.sorttable_tbody);
                    this.className = this.className.replace('sorttable_sorted', 'sorttable_sorted_reverse');
                    this.removeChild(document.getElementById('sorttable_sortfwdind'));

                    var sortrevind = document.createElement('span');
                        sortrevind.id = "sorttable_sortrevind";
                        sortrevind.textContent = '\xA0\u25B4';
                    this.appendChild(sortrevind);
                    return;
                }
                if (this.className.search(/\bsorttable_sorted_reverse\b/) != -1)
                {
                    // if we're already sorted by this column in reverse, just re-reverse the table, which is quicker
                    sorttable.reverse(this.sorttable_tbody);
                    this.className = this.className.replace('sorttable_sorted_reverse', 'sorttable_sorted');
                    this.removeChild(document.getElementById('sorttable_sortrevind'));

                    var sortfwdind = document.createElement('span');
                        sortfwdind.id = "sorttable_sortfwdind";
                        sortfwdind.textContent = '\xA0\u25BE';
                    this.appendChild(sortfwdind);
                    return;
                }

                // remove sorttable_sorted classes
                var theadrow = this.parentNode;
                for (var c = 0; c < theadrow.childNodes.length; c++)
                {
                    var cell = theadrow.childNodes[c];
                    if (cell.nodeType == 1)
                    {
                        cell.className = cell.className.replace('sorttable_sorted_reverse','');
                        cell.className = cell.className.replace('sorttable_sorted','');
                    }
                }

                sortfwdind = document.getElementById('sorttable_sortfwdind');
                if (sortfwdind)
                {
                    sortfwdind.parentNode.removeChild(sortfwdind);
                }
                sortrevind = document.getElementById('sorttable_sortrevind');
                if (sortrevind)
                {
                    sortrevind.parentNode.removeChild(sortrevind);
                }
                this.className += ' sorttable_sorted';

                sortfwdind = document.createElement('span');
                sortfwdind.id = "sorttable_sortfwdind";
                sortfwdind.textContent = '\xA0\u25BE';
                this.appendChild(sortfwdind);

    	        // build an array to sort. This is a Schwartzian transform thing, i.e., we "decorate" each row with the actual sort key,
	            // sort based on the sort keys, and then put the rows back in order which is a lot faster because you only do getInnerText once per row
        	    var row_array = [];
	            var col = this.sorttable_columnindex;
    	        var rows = this.sorttable_tbody.rows;
	            for (var j=0; j<rows.length; j++)
                {
	                row_array[row_array.length] = [sorttable.getInnerText(rows[j].cells[col]), rows[j]];
    	        }
	            row_array.sort(this.sorttable_sortfunction);

                var tb = this.sorttable_tbody;
    	        for (var k=0; k<row_array.length; k++)
                {
    	            tb.appendChild(row_array[k][1]);
    	        }
	            delete row_array;

            }, false); // eventListener
        }
    },

    guessType: function(table, column)
    {
        // guess the type of a column based on its first non-blank row
        var sortfn = sorttable.sort_alpha;
        for (var i=0; i < table.tBodies[0].rows.length; i++)
        {
            var text = sorttable.getInnerText(table.tBodies[0].rows[i].cells[column]);
            if (text !== '')
            {
                if (text.match(/^-?[Ј$¤]?[\d,.]+%?$/))
                {
                    return sorttable.sort_numeric;
                }
                // check for a date: dd/mm/yyyy or dd/mm/yy
                // can have / or . or - as separator
                // can be mm/dd as well
                var possdate = text.match(sorttable.DATE_RE);
                if (possdate)
                {
                    // looks like a date
                    var first = parseInt(possdate[1], 10);
                    var second = parseInt(possdate[2], 10);
                    if (first > 12)
                    {
                        // definitely dd/mm
                        return sorttable.sort_ddmm;
                    }
                    else if (second > 12)
                    {
                        return sorttable.sort_mmdd;
                    }
                    else
                    {
                        // looks like a date, but we can't tell which, so assume that it's dd/mm (English imperialism!) and keep looking
                        sortfn = sorttable.sort_ddmm;
                    }
                }
            }
        }
        return sortfn;
    },
  
    getInnerText: function(node)
    {
        // gets the text we want to use for sorting for a cell.
        // strips leading and trailing whitespace.
        // this is *not* a generic getInnerText function; it's special to sorttable.
        // for example, you can override the cell text with a customkey attribute.
        // it also gets .value for <input> fields.

        var hasInputs = node.getElementsByTagName('input').length;

        if (node.getAttribute("sorttable_customkey") !== null)
        {
            return node.getAttribute("sorttable_customkey");
        }
        else if (typeof node.textContent != 'undefined' && !hasInputs)
        {
            return node.textContent.trim();
        }
        else if (typeof node.innerText != 'undefined' && !hasInputs)
        {
            return node.innerText.trim();
        }
        else if (typeof node.text != 'undefined' && !hasInputs)
        {
            return node.text.trim();
        }
        else
        {
            switch (node.nodeType)
            {
                case 3:
                    if (node.nodeName.toLowerCase() == 'input')
                    {
                        return node.value.trim();
                    }
                case 4:
                    return node.nodeValue.trim();
                case 1:
                case 11:
                    var innerText = '';
                    for (var i = 0; i < node.childNodes.length; i++)
                    {
                        innerText += sorttable.getInnerText(node.childNodes[i]);
                    }
                    return innerText.trim();
                default:
                    return '';
           }
        }
        return '';
    },

    reverse: function(tbody)
    {
        // reverse the rows in a tbody
        var newrows = [];
        for (var i = 0; i < tbody.rows.length; i++)
        {
            newrows[newrows.length] = tbody.rows[i];
        }
        for (var j=newrows.length-1; j >= 0; j--)
        {
           tbody.appendChild(newrows[j]);
        }
        delete newrows;
    },

    /* sort functions; each sort function takes two parameters, a and b,  you are comparing a[0] and b[0] */

    sort_numeric: function(a, b)
    {
        var aa = parseFloat(a[0].replace(/[^0-9.-]/g,''));
        if (isNaN(aa)) aa = 0;
        var bb = parseFloat(b[0].replace(/[^0-9.-]/g,''));
        if (isNaN(bb)) bb = 0;
        return aa-bb;
    },

    sort_alpha: function(a, b)
    {
        if (a[0] == b[0]) return 0;
        if (a[0] < b[0]) return -1;
        return 1;
    },

    sort_ddmm: function(a, b)
    {
        var mtch = a[0].match(sorttable.DATE_RE);
        var y = mtch[3];
        var m = mtch[2];
        var d = mtch[1];
        if (m.length == 1) m = '0' + m;
        if (d.length == 1) d = '0' + d;
        var dt1 = y + m + d;

        mtch = b[0].match(sorttable.DATE_RE);
        y = mtch[3];
        m = mtch[2];
        d = mtch[1];
        if (m.length == 1) m = '0' + m;
        if (d.length == 1) d = '0' + d;
        var dt2 = y + m + d;

        if (dt1 == dt2) return 0;
        if (dt1 < dt2) return -1;
        return 1;
    },

    sort_mmdd: function(a, b)
    {
        var mtch = a[0].match(sorttable.DATE_RE);
        var y = mtch[3];
        var d = mtch[2];
        var m = mtch[1];
        if (m.length == 1) m = '0' + m;
        if (d.length == 1) d = '0' + d;
        var dt1 = y + m + d;

        mtch = b[0].match(sorttable.DATE_RE);
        y = mtch[3];
        d = mtch[2];
        m = mtch[1];
        if (m.length == 1) m = '0' + m;
        if (d.length == 1) d = '0' + d;
        var dt2 = y + m + d;

        if (dt1 == dt2) return 0;
        if (dt1 < dt2) return -1;
        return 1;
    }
};

if (document.addEventListener)
{
    document.addEventListener("DOMContentLoaded", sorttable.init, false);
}